//*
//*  -- LAPACK routine (version 3.0) --
//*     Univ. of Tennessee, Univ. of California Berkeley, NAG Ltd.,
//*     Courant Institute, Argonne National Lab, and Rice University
//*     March 31, 1993
//*
//*     .. Scalar Arguments ..
//INTEGER            INFO, LDA, M, N
//*     ..
//*     .. Array Arguments ..
//INTEGER            IPIV( * )
//DOUBLE PRECISION   A( LDA, * )
//*     ..
//*
//*  Purpose
//*  =======
//*
//*  DGETRF computes an LU factorization of a general M-by-N matrix A
//*  using partial pivoting with row interchanges.
//*
//*  The factorization has the form
//*     A = P * L * U
//*  where P is a permutation matrix, L is lower triangular with unit
//*  diagonal elements (lower trapezoidal if m > n), and U is upper
//*  triangular (upper trapezoidal if m < n).
//*
//*  This is the right-looking Level 3 BLAS version of the algorithm.
//*
//*  Arguments
//*  =========
//*
//*  M       (input) INTEGER
//*          The number of rows of the matrix A.  M >= 0.
//*
//*  N       (input) INTEGER
//*          The number of columns of the matrix A.  N >= 0.
//*
//*  A       (input/output) DOUBLE PRECISION array, dimension (LDA,N)
//*          On entry, the M-by-N matrix to be factored.
//*          On exit, the factors L and U from the factorization
//*          A = P*L*U; the unit diagonal elements of L are not stored.
//*
//*  LDA     (input) INTEGER
//*          The leading dimension of the array A.  LDA >= max(1,M).
//*
//*  IPIV    (output) INTEGER array, dimension (min(M,N))
//*          The pivot indices; for 1 <= i <= min(M,N), row i of the
//*          matrix was interchanged with row IPIV(i).
//*
//*  INFO    (output) INTEGER
//*          = 0:  successful exit
//*          < 0:  if INFO = -i, the i-th argument had an illegal value
//*          > 0:  if INFO = i, U(i,i) is exactly zero. The factorization
//*                has been completed, but the factor U is exactly
//*                singular, and division by zero will occur if it is used
//*                to solve a system of equations.

#include "dsyev.h"
#include "dgetrf.h"
//#include <cmath>
//#include <iostream>
//
//using namespace std;


void LapackLib::dgetrf( int m, int n, double **A, double **L, double **U )
{
	int lda, lpiv, info;
	int *ipiv;
	double *a;

	lda = m; // The leading dimension of the matrix to be solved.

	lpiv = (m < n) ? m : n;
	ipiv = new int[lpiv]; /* The output pivots */

	/* Here we convert the incoming array, assumed to be in double index C form, to a Fortran style matrix. */
	// The Fortran format is column-major order.
	a = new double[m*n];
	for (int row = 0; row < m; ++row) {
		for (int col = 0; col < n; ++col) {
			a[row+col*m] = A[row][col];
		}
	}

	// Run the LU factorization routine
	dgetrf_(&m, &n, a, &lda, ipiv, &info);

	// Extract the result (Note: the unit diagonal elements of L are not stored)
	// The Fortran format is column-major order.
	for (int row = 0; row < m; ++row) {
		for (int col = 0; col < n; ++col) {
			if (row > col) {
				L[row][col] = a[row+col*m];
			}
			else {
				if (row == col) {
					L[row][col] = 1.0;
				}
				U[row][col] = a[row+col*m];
			}
		}
	}

	// Note that unlike MATLAB, we do not permute the rows of L according to ipiv.

	//cout << "result: " << info << endl;

	delete a;
	delete ipiv;
}
