/*
  In this header file, I have defined a simplified function
  call to the ARPACK solver routine for a simple, symmetric
  eigenvalue problem Av = lv.  The looping procedure and
  final extraction of eigenvalues and vectors is handled
  automatically.  Most of the parameters to the FORTRAN
  functions are hidden from the user, since most of them are
  determined from user input anyway.
  
  The remaining parameters to the function calls are as follows:
  
    dsaupd(int n, int nev, double *Evals)
    dsaupd(int n, int nev, double *Evals, double **Evecs)

    n: the order of the square matrix A
    nev: the number of eigenvalues to be found, starting at the
         bottom.  Note that the highest eigenvalues, or some
	 other choices, can be found.  For now, the choice of
	 the lowest nev eigenvalues is hard-coded.
    Evals: a one-dimensional array of length nev to hold the
           eigenvalues.
    Evecs: a two-dimensional array of size nev by n to hold the
           eigenvectors.  If this argument is not provided, the
	   eigenvectors are not calculated.  Note that the
	   vectors are stored as columns of Evecs, so that the
	   elements of vector i are the values Evecs[j][i].

  The function is overloaded, so that you can call it in either
  fashion and the appropriate version will be run.

  To use these function calls, there must be a function
  defined in the calling program of the form

    av(int n, double *in, double *out)

  where the function av finds out = A.in, and n is the order of
  the matrix A.  This function must be defined before the
  statement that includes this header file, as it needs to know
  about this function.  It is used in the looping procedure.

  Scot Shaw
  30 August 1999
*/

extern "C" {
	void dsaupd_(int *ido, char *bmat, int *n, char *which,
			int *nev, double *tol, double *resid, int *ncv,
			double *v, int *ldv, int *iparam, int *ipntr,
			double *workd, double *workl, int *lworkl,
			int *info);

	void dseupd_(int *rvec, char *All, int *select, double *d,
			double *v1, int *ldv1, double *sigma, 
			char *bmat, int *n, char *which, int *nev,
			double *tol, double *resid, int *ncv, double *v2,
			int *ldv2, int *iparam, int *ipntr, double *workd,
			double *workl, int *lworkl, int *ierr);
};

/**
 * Checks to see if the given matrix has the correct symmetric format.
 * The matrix must be symmetric, lower triangular, and have data type TAUCS_DOUBLE.
 * @param[in] mat Matrix in taucs_ccs format.
 * @return Boolean with the test result.
 */
bool CheckSymmetric( const taucs_ccs_matrix *mat );

/**
* Multiplies matA by x and stores the result in b.
* Assumes all memory has been allocated and the sizes match;
* also assumes matA is symmetric!!
* @param[in] matA Symmetric matrix in taucs_ccs format.
* @param[out] x Vector to multiply on the right of matA.
* @param[out] b Vector to output the result.
*/
void MulSymmMatrixVector( const taucs_ccs_matrix *matA, const taucsType *x, taucsType *b );

/**
 * Interface for ARPACK's dsaupd_ and dseupd_ to find the eigenvalues
 * (and eigenvectors) of a real symmetric matrix in taucs_ccs format.
 * You can ask which eigenvalues to output, using the "which" parameter.
 * Here the possible options are:
 *   LM: largest magnitude
 *   SM: smallest magnitude
 *   LA: largest real component
 *   SA: smallest real component
 *   LI: largest imaginary component
 *   SI: smallest imaginary component
 * This function uses the MulSymmMatrixVector routine
 * to perform matrix-vector multiplications.
 * @param[in] mat Symmetric matrix in taucs_ccs format.
 * @param[in] nev Number of eigenvalues (and eigenvectors) to compute.
 * @param[in] evType Which eigenvalues to output. Possible options are LM, SM, LA, SA, LI, SI.
 * @param[out] Evals The resulting eigenvalues.
 * @param[out] Evecs (Optional) The resulting eigenvectors. A value of NULL makes the routine not compute the eigenvectors.
 */
void dsaupd( const taucs_ccs_matrix* mat, int nev, char *which, double *Evals, double **Evecs = NULL );
