#include "dsyev.h"
#include <cmath>

void LapackLib::dsyev(double **H, int n, double *E)
{
	char jobz, uplo;
	int lda, lwork, info;
	double *a, *work, **Evecs;

	jobz = 'N'; /* V/N indicates that eigenvectors should/should not
				be calculated. */

	uplo = 'U'; /* U/L indicated that the upper/lower triangle of the
				symmetric matrix is stored. */

	lda = n; // The leading dimension of the matrix to be solved.

	lwork = 3*n-1;
	work = new double[lwork]; /* The work array to be used by dsyev and
							  its size. */

	a = dsyev_ctof(H, n, lda); /* Here we convert the incoming array, assumed
							   to be in double index C form, to a Fortran
							   style matrix. */

	dsyev_(&jobz, &uplo, &n, a, &lda, E, work, &lwork, &info);

	Evecs = dsyev_ftoc(a, n, lda);
	dsyev_sort(E, Evecs, n); /* Though it is just a dummy array, we convert
							 the returned values for eigenvectors from
							 Fortran single-pointer form to C double-
							 pointer form.  Then we sort the results by
							 eigenvalue in increasing order. */

	delete a;
	delete work;
	delete Evecs;
}


void LapackLib::dsyev(double **H, int n, double *E, double **Evecs)
{
	char jobz, uplo;
	int lda, lwork, info;
	double *a, *work;

	jobz = 'V';
	uplo = 'U';
	lda = n;

	lwork = 3*n-1;
	work = new double[lwork];

	a = dsyev_ctof(H, n, lda);

	dsyev_(&jobz, &uplo, &n, a, &lda, E, work, &lwork, &info);

	dsyev_ftoc2(a, Evecs, n, lda);
	dsyev_sort(E, Evecs, n);

	delete a;
	delete work;
}


double* LapackLib::dsyev_ctof(double **in, int rows, int cols)
{
	double *out;
	int i, j;

	out = new double[rows*cols];
	for (i=0; i<rows; i++) for (j=0; j<cols; j++) out[i+j*cols] = in[i][j];
	return(out);
}


double** LapackLib::dsyev_ftoc(double *in, int rows, int cols)
{
	double **out;
	int i, j;

	out = new double*[rows];
	for (i=0; i<rows; i++) out[i] = new double[cols];
	for (i=0; i<rows; i++) for (j=0; j<cols; j++) out[i][j] = in[i+j*cols];
	return(out);
}


void LapackLib::dsyev_ftoc2(double *in, double **out, int rows, int cols)
{
	int i, j;

	for (i=0; i<rows; i++) for (j=0; j<cols; j++) out[i][j] = in[i+j*cols];
}


void LapackLib::dsyev_sort(double *E, double **Evecs, int N)
{
	double temp;
	int i, j, k;

	for (j=0; j<N; j++) for (i=0; i<N-1; i++)
		if (fabs(E[i])>fabs(E[i+1])) {
			temp = E[i]; E[i] = E[i+1]; E[i+1] = temp;

			for (k=0; k<N; k++) {
				temp = Evecs[k][i];
				Evecs[k][i] = Evecs[k][i+1];
				Evecs[k][i+1] = temp;
			}
		}
}

////
/////*
////  This is an example program using the header file
////  dsyev.h, my implementation of the LAPACK symmetric matrix
////  solver routine.  See that header file for more
////  documentation on its use.
////  
////  For this example, we will work with the four by four
////  matrix H[i,j] = i^j + j^i.  This is symmetric, so the
////  eigenvalues are real.  The output of this program
////  should be:
////
////  Eigenvalue  0: 0.151995
////  Eigenvector 0: (0.610828, -0.72048, 0.324065, -0.0527271)
////  Eigenvalue  1: 1.80498
////  Eigenvector 1: (-0.7614, -0.42123, 0.481884, -0.103072)
////  Eigenvalue  2: 17.6352
////  Eigenvector 2: (0.216885, 0.547084, 0.764881, -0.26195)
////  Eigenvalue  3: 556.408
////  Eigenvector 3: (0.0110019, 0.064609, 0.278795, 0.958112)
////
////  Scot Shaw
////  7 September 1999
////*/
////
////// Begin with some standard include files.
////#include <iostream>
////#include "dsyev.h"
////
////using namespace std;
////using namespace LapackLib;
////
////void main(int nargs, char *args[])
////{
////  double **H, *Evals, **Evecs;
////  int i, j;
////  
////  int n=4;
////  
////  // Generate the matrix for my equation
////  
////  H = new double*[n];
////  for (i=0; i<n; i++) H[i] = new double[n];
////  for (i=0; i<n; i++) for (j=0; j<n; j++)
////    H[i][j] = pow((double) i+1, j+1) + pow((double) j+1, i+1);
////  
////  // Call the LAPACK solver
////  
////  Evals = new double[n]; 
////  Evecs = new double*[n];
////  for (i=0; i<n; i++) Evecs[i] = new double[n];
////
////  dsyev(H, n, Evals, Evecs);
////  
////  // Output the results
////
////  for (i=0; i<n; i++) {
////    cout << "Eigenvalue  " << i << ": " << Evals[i] << "\n";
////    cout << "Eigenvector " << i << ": ("
////	 << Evecs[0][i] << ", "
////	 << Evecs[1][i] << ", "
////	 << Evecs[2][i] << ", "
////	 << Evecs[3][i] << ")\n";
////  }
////}
////

