#ifndef XMLPARAM_H
#define XMLPARAM_H

#include "libxmlparam_global.h"
#include <QDomDocument>
#include <QDomNodeList>
#include <string>
#include <vector>


// Convenient type definitions for lists of quantities
typedef std::vector<bool> BoolArray;
typedef std::vector<int> IntArray;
typedef std::vector<float> FloatArray;
typedef std::vector<std::string> StringArray;


/**
 * A class for loading / saving global parameters in XML format.
 *
 *
 *	How to use this library:
 *
 *  In your main() function, pass the command-line arguments and
 *  a variable storing the parameters (xmlParam par) into the method
 *
 *      bool initParamXML(int argc, char *argv[], xmlParam &par);
 *
 *  This method returns true if the reading/writing of parameters is completed;
 *  it will return false if there is some error.
 *
 *  Also, declare and implement a function to link global variables to the
 *  parameters structure:
 *
 *	void setXMLParamList(xmlParam &p) {
 *		// populate the parameter list with your desired global variables
 *		p.addParam("endFrame", "int", (void*)&endFrame);
 *		p.addParam("exposure", "float", (void*)&exposure);
 *		p.addParam("basePath", "string", (void*)&basePath);
 *	}
 *
 *  Allowed types are: bool, int, float, string, boolArray, intArray, floatArray, stringArray.
 *
 *
 */


/**
 * A class for loading / saving global parameters in XML format.
 */
class LIBXMLPARAM_EXPORT xmlParam {
public:
	// Default Constructor
	xmlParam();

	// Constructor with name
	xmlParam(char *name);

	// Helper Methods
	void addParam(std::string name, std::string type, void* ptr, const char *comment = NULL);
	void resetParams();
	void printParams();
	void printParamsINI();
	QDomDocument makeParamXML();
	bool writeParamXML(const char* filename, QDomDocument &doc);
	bool readParamXML(const char* filename, QDomDocument &doc);
	void loadParamXML(QDomDocument &doc);
	std::string getTopName();
	bool initParamXML(int argc, char *argv[]);

private:
	void printUsage(char *argv0);
	std::vector<std::string> paramList;
	std::vector<std::string> paramTypes;
	std::vector<void*> paramPtr;
	std::vector<std::string> paramComment;
	std::string topName;
};


// Declaration of setXMLParamList (this must be defined and implemented in your main.cpp file!)
extern void setXMLParamList(xmlParam &p);


#endif // XMLPARAM_H
