/*
  This is an example program using the header file
  dsaupd.h, my implementation of the ARPACK sparse matrix
  solver routine.  See that header file for more
  documentation on its use.
  
  For this example, we will work with the four by four
  matrix H[i,j] = i^j + j^i.  The output of this program
  should be:

  Eigenvalue  0: 0.151995
  Eigenvector 0: (0.610828, -0.72048, 0.324065, -0.0527271)
  Eigenvalue  1: 1.80498
  Eigenvector 1: (0.7614, 0.42123, -0.481884, 0.103072)
  Eigenvalue  2: 17.6352
  Eigenvector 2: (-0.216885, -0.547084, -0.764881, 0.26195)
  
  Scot Shaw
  7 September 1999
*/

#include <iostream>
//#include <fstream>
#include <cmath>
//#include <cstdio>

#ifndef WIN32
#include <unistd.h>
#include <pthread.h>
#endif

#define TAUCS_CORE_DOUBLE
extern "C" {
#include <taucs.h>
}

typedef double taucsType;

using namespace std;

///*
//  To use the routines from dsaupd.h, we need to define our
//  own matrix multiplication routine.  Here I show one method
//  that would exploit sparsity by storing only the non-zero
//  matrix elements (though for this example we won't actually
//  have a sparse matrix to work with).
//
//  There is a routine for the multiplication, and a global
//  variable T that will hold the non-zero matrix elements
//  and their indicies.  The global integer tlen will hold the
//  number of non-zero elements.  The matrix multiplication function
//  needs to have the form below, and must be defined before
//  the file dsaupd.h is included.
//*/
//double **T;
//int tlen;
//void av(int n, double *in, double *out);
//void av(int n, double *in, double *out)
//{
//  int i;
//
//  for (i=0; i<n; i++) out[i] = 0;
//  for (i=0; i<tlen; i++) out[(int)T[i][0]] += in[(int)T[i][1]] * T[i][2];
//}

#include "dsaupd.h"

void dsaupd( const taucs_ccs_matrix* mat, int nev, char *which, double *Evals, double **Evecs /*= NULL */ )
{
	if (!CheckSymmetric(mat)) {
		return;
	}

	int n = mat->n;			/**< Size of the symmetric matrix. */

	int ido = 0;			/*  Initialization of the reverse communication
								parameter. */

	char bmat[2] = "I";		/*  Specifies that the right hand side matrix
								should be the identity matrix; this makes
								the problem a standard eigenvalue problem.
								Setting bmat = "G" would have us solve the
								problem Av = lBv (this would involve using
								some other programs from BLAS, however). */

	////char *which = evType;	/*  Ask for the nev eigenvalues of smallest
	////							magnitude.  The possible options are
	////							LM: largest magnitude
	////							SM: smallest magnitude
	////							LA: largest real component
	////							SA: smallest real compoent
	////							LI: largest imaginary component
	////							SI: smallest imaginary component */

	double tol = 0.0;		/*  Sets the tolerance; tol<=0 specifies 
								machine precision */

	double *resid;
	resid = new double[n];

	int ncv = 4*nev;		/*  The largest number of basis vectors that will
								be used in the Implicitly Restarted Arnoldi
								Process.  Work per major iteration is
								proportional to N*NCV*NCV. */
	if (ncv>n) ncv = n;

	double *v;
	int ldv = n;
	v = new double[ldv*ncv];

	int *iparam;
	iparam = new int[11];	/*  An array used to pass information to the routines
								about their functional modes. */
	iparam[0] = 1;			// Specifies the shift strategy (1->exact)
	iparam[2] = 3*n;		// Maximum number of iterations
	iparam[6] = 1;			/*  Sets the mode of dsaupd.
								1 is exact shifting,
								2 is user-supplied shifts,
								3 is shift-invert mode,
								4 is buckling mode,
								5 is Cayley mode. */

	int *ipntr;
	ipntr = new int[11];	/*  Indicates the locations in the work array workd
								where the input and output vectors in the
								callback routine are located. */

	double *workd;
	workd = new double[3*n];

	double *workl;
	workl = new double[ncv*(ncv+8)];

	int lworkl = ncv*(ncv+8);		/* Length of the workl array */

	int info = 0;					/*  Passes convergence information out of the iteration
										routine. */

	int rvec = (Evecs != NULL);		/* Specifies that eigenvectors should not be calculated */

	int *select;
	select = new int[ncv];
	double *d;
	d = new double[2*ncv];	/*  This vector will return the eigenvalues from
								the second routine, dseupd. */
	double sigma;
	int ierr;

	/* Here we enter the main loop where the calculations are
	performed.  The communication parameter ido tells us when
	the desired tolerance is reached, and at that point we exit
	and extract the solutions. */

	do {
		dsaupd_(&ido, bmat, &n, which, &nev, &tol, resid, 
			&ncv, v, &ldv, iparam, ipntr, workd, workl,
			&lworkl, &info);

		if ((ido==1)||(ido==-1)) {
			//av(n, workd+ipntr[0]-1, workd+ipntr[1]-1);
			MulSymmMatrixVector(mat, workd+ipntr[0]-1, workd+ipntr[1]-1);
		}
	} while ((ido==1)||(ido==-1));

	/* From those results, the eigenvalues and vectors are
	extracted. */

	if (info<0) {
		cout << "Error with dsaupd, info = " << info << "\n";
		cout << "Check documentation in dsaupd\n\n";
	} else {
		dseupd_(&rvec, "All", select, d, v, &ldv, &sigma, bmat,
			&n, which, &nev, &tol, resid, &ncv, v, &ldv,
			iparam, ipntr, workd, workl, &lworkl, &ierr);

		if (ierr!=0) {
			cout << "Error with dseupd, info = " << ierr << "\n";
			cout << "Check the documentation of dseupd.\n\n";
		} else if (info==1) {
			cout << "Maximum number of iterations reached.\n\n";
		} else if (info==3) {
			cout << "No shifts could be applied during implicit\n";
			cout << "Arnoldi update, try increasing NCV.\n\n";
		}

		/* Before exiting, we copy the solution information over to
		the arrays of the calling program, then clean up the
		memory used by this routine. */

		if (rvec) {
			for (int i=0; i<nev; ++i) {
				Evals[i] = d[i];
			}

			for (int i=0; i<nev; ++i) {
				for (int j=0; j<n; ++j) {
					Evecs[i][j] = v[i*n+j];
				}
			}
		}
		else {
			for (int i=0; i<nev; ++i) {
				Evals[i] = d[i];
			}
		}

		delete resid;
		delete v;
		delete iparam;
		delete ipntr;
		delete workd;
		delete workl;
		delete select;
		delete d;
	}
}


bool CheckSymmetric( const taucs_ccs_matrix *mat )
{
	// Check to see if mat is symmetric, double, and lower triangular.
	if ((mat->flags & TAUCS_DOUBLE) &&
		(mat->flags & TAUCS_SYMMETRIC) &&
		(mat->flags & TAUCS_LOWER) &&
		(mat->n == mat->m))
	{
		return true;
	}
	return false;
}

void MulSymmMatrixVector( const taucs_ccs_matrix *matA, const taucsType *x, taucsType *b )
{
	if (!CheckSymmetric(matA)) {
		return;
	}

	// Make b all zero
	memset(b, 0, matA->m * sizeof(taucsType));

	for (int col = 0; col < matA->n; ++col) {
		// going over column col of matA, multiplying
		// it by x[col] and setting the appropriate values
		// of vector b
		for (int p = matA->colptr[col]; p < matA->colptr[col+1]; ++p) {
			int row = matA->rowind[p];
			b[row] += x[col] * matA->taucs_values[p];

			// That was for the lower triangle, now also
			// multiple the upper triangle with x
			if (row != col) {
				b[col] += x[row] * matA->taucs_values[p];
			}
		}
	}
}



//void dsaupd(int n, int nev, double *Evals, double **Evecs)
//{
//	int ido = 0;
//	char bmat[2] = "I";
//	char which[3] = "LM";
//	double tol = 0.0;
//	double *resid;
//	resid = new double[n];
//	int ncv = 4*nev;
//	if (ncv>n) ncv = n;
//	double *v;
//	int ldv = n;
//	v = new double[ldv*ncv];
//	int *iparam;
//	iparam = new int[11];
//	iparam[0] = 1;
//	iparam[2] = 3*n;
//	iparam[6] = 1;
//	int *ipntr;
//	ipntr = new int[11];
//	double *workd;
//	workd = new double[3*n];
//	double *workl;
//	workl = new double[ncv*(ncv+8)];
//	int lworkl = ncv*(ncv+8);
//	int info = 0;
//	int rvec = 1;  // Changed from above
//	int *select;
//	select = new int[ncv];
//	double *d;
//	d = new double[2*ncv];
//	double sigma;
//	int ierr;
//
//	do {
//		dsaupd_(&ido, bmat, &n, which, &nev, &tol, resid, 
//			&ncv, v, &ldv, iparam, ipntr, workd, workl,
//			&lworkl, &info);
//
//		if ((ido==1)||(ido==-1)) {
//			//av(n, workd+ipntr[0]-1, workd+ipntr[1]-1);
//			MulSymmMatrixVector(mat, workd+ipntr[0]-1, workd+ipntr[1]-1);
//		}
//	} while ((ido==1)||(ido==-1));
//
//	if (info<0) {
//		cout << "Error with dsaupd, info = " << info << "\n";
//		cout << "Check documentation in dsaupd\n\n";
//	} else {
//		dseupd_(&rvec, "All", select, d, v, &ldv, &sigma, bmat,
//			&n, which, &nev, &tol, resid, &ncv, v, &ldv,
//			iparam, ipntr, workd, workl, &lworkl, &ierr);
//
//		if (ierr!=0) {
//			cout << "Error with dseupd, info = " << ierr << "\n";
//			cout << "Check the documentation of dseupd.\n\n";
//		} else if (info==1) {
//			cout << "Maximum number of iterations reached.\n\n";
//		} else if (info==3) {
//			cout << "No shifts could be applied during implicit\n";
//			cout << "Arnoldi update, try increasing NCV.\n\n";
//		}
//
//		int i, j;
//		for (i=0; i<nev; i++) Evals[i] = d[i];
//		for (i=0; i<nev; i++) for (j=0; j<n; j++) Evecs[j][i] = v[i*n+j];
//
//		delete resid;
//		delete v;
//		delete iparam;
//		delete ipntr;
//		delete workd;
//		delete workl;
//		delete select;
//		delete d;
//	}
//}
//
//void main()
//{
//  int n, nev, i;
//  double **Evecs, *Evals;
//  
//  ////n = 4; // The order of the matrix
//
//  /////*
//  ////  Here we generate the matrix T for the multiplication.
//  ////  It helps if we know the number of non-zero matrix
//  ////  elements before we find them, so that we can save on
//  ////  storage space.  If not, generate enough storage space
//  ////  for T to cover an upper limit.
//  ////*/
//
//  ////tlen = n*n;
//  ////T = new double*[tlen];
//  ////for (i=0; i<tlen; i++) T[i] = new double[3];
//
//  ////tlen = 0;
//  ////for (i=0; i<n; i++) for (j=0; j<n; j++) {
//  ////  T[tlen][0] = i;
//  ////  T[tlen][1] = j;
//  ////  T[tlen][2] = pow((double)i+1, j+1) + pow((double)j+1, i+1);
//  ////  tlen++;
//  ////}
//
//  // Generate a matrix to find its eigenvalues / eigenvectors.
//  n = 4;
//  int predictedNNZ = n*(n+1)/2;
//  taucs_ccs_matrix *mat = taucs_ccs_create(n, n, predictedNNZ, TAUCS_DOUBLE|TAUCS_SYMMETRIC|TAUCS_LOWER);
//  if (!mat) {
//	  cerr << "Could not allocate taucs_ccs matrix!" << endl;
//	  return;
//  }
//
//  int nnz = 0;
//  for (int col = 0; col < n; ++col) {
//	  mat->colptr[col] = nnz;
//	  for (int row = col; row < n; ++row) {
//		  mat->rowind[nnz] = row;
//		  mat->taucs_values[nnz] = pow((double)col+1, row+1) + pow((double)row+1, col+1);
//		  ++nnz;
//	  }
//  }
//  mat->colptr[n] = nnz;
//
//  // Print out the matrix
//  cout << "Matrix dimensions (symmetric): " << mat->m << " by " << mat->n << " with "
//	   << mat->colptr[mat->n] << "(" << predictedNNZ << ") nonzero entries" << endl;
//  cout << "values:";
//  for (int i = 0; i < nnz; ++i) {
//	  cout << " " << mat->taucs_values[i];
//  }
//  cout << endl << "rowind:";
//  for (int i = 0; i < nnz; ++i) {
//	  cout << " " << mat->rowind[i];
//  }
//  cout << endl << "colptr: ";
//  for (int i = 0; i < n+1; ++i) {
//	  cout << " " << mat->colptr[i];
//  }
//  cout << endl;
//
//  //taucs_ccs_matrix *mat = taucs_ccs_read_ijv("C:\\Mij_sparse.txt", TAUCS_DOUBLE|TAUCS_SYMMETRIC|TAUCS_LOWER);
//  //n = mat->n;
//  //cout << "Matrix dimensions (symmetric): " << mat->m << "x" << mat->n << " ("
//  //  << mat->colptr[mat->n] << " nnz)" << endl;
//
//
//  /*
//    We will calculate both the eigenvalues and the
//    eigenvectors in this example.  To not calculate the
//    vectors, just leave that argument out of the call to
//    dsaupd.
//
//    We will find only three of the eigenvalues, to make
//    it more clear how the eigenvectors come out of the
//    routine.  Besides that, the ARPACK routine won't
//    calculate all of the eigenvalues.  I'm not sure if
//    that is intentional, or if I am missing something.
//  */
//
//  nev = 6; // The number of values to calculate
//
//  if (nev > n) {
//	  nev = n-1;
//  }
//
//  Evals = new double[nev];
//  Evecs = new double*[nev];
//  for (i=0; i<nev; i++) Evecs[i] = new double[n];
//
//  dsaupd(mat, nev, "LM", Evals);
//  for (i=0; i<nev; i++) {
//	  cout << "Eigenvalue  " << i << ": " << Evals[i] << "\n";
//  }
//
//  dsaupd(mat, nev, "LM", Evals, Evecs);
//
//  // Now we output the results.
//  for (i=0; i<nev; i++) {
//    cout << "Eigenvalue  " << i << ": " << Evals[i] << "\n";
//      cout << "Eigenvector " << i << ": ("
//      << Evecs[i][0] << ", "
//      << Evecs[i][1] << ", "
//      << Evecs[i][2] << ", "
//      << Evecs[i][3] << ")\n";
//  }
//
//
//  // Multiplication test: try multiplying with the eigenvector
//  int tryIdx = 0;
//  MulSymmMatrixVector(mat, Evecs[tryIdx], Evecs[1]);
//  cout << "Result:" << endl;
//  cout << "Eigenvector " << i << ": ("
//	  << Evecs[1][0]/Evals[tryIdx] << ", "
//	  << Evecs[1][1]/Evals[tryIdx] << ", "
//	  << Evecs[1][2]/Evals[tryIdx] << ", "
//	  << Evecs[1][3]/Evals[tryIdx] << ")\n";
//}
